import json

from fastapi import APIRouter, Depends
from fastapi.responses import JSONResponse

from bsition.api.models.document import Access, Document, DocumentUpdate
from bsition.api.utils.security import get_current_user
from bsition.backend.elastic import search as elastic
from bsition.backend.mongo import documents as mongo
from bsition.backend.postgres import relations as postgres

router = APIRouter()


@router.post("")
def create(aux: Document, user: tuple = Depends(get_current_user)):
    doc_id = mongo.create_document(
        {
            "name": aux.name,
            "owner": user[0],
            "data": aux.data if aux.data is not None else "",
        }
    )
    postgres.give_access_doc(user[0], doc_id, 1)
    return JSONResponse(
        content={"detail": "Document created", "id": doc_id}, status_code=201
    )


@router.get("")
def get_documents(query: str = None, user: tuple = Depends(get_current_user)):
    if query is not None:
        return elastic.search("test-index", query, user[0])
    return mongo.get_documents(user[0])


@router.get("/{id}")
def get_document(id: str):
    return mongo.get_document_by_id(id)


@router.put("/{id}")
def edit_data(aux: DocumentUpdate, id: str):
    if aux.data is not None:
        mongo.edit_data(id, aux.data)
    if aux.access is not None:
        mongo.edit_access(id, aux.access)
    if aux.public is not None:
        print("public")
        mongo.make_public(id, aux.public)
    if aux.name is not None:
        mongo.edit_name(id, aux.name)
    return JSONResponse(content={"detail": "Document updated."}, status_code=202)


@router.delete("/{id}")
def delete_doc(id: str):
    mongo.delete_document(id)
    return JSONResponse(content={"detail": "Document deleted."}, status_code=201)


@router.get("/{id}/access")
def get_doc_access(id: str):
    return postgres.access_to_doc(id)


@router.get("/{id}/access/me")
def get_access(id: str, user: tuple = Depends(get_current_user)):
    return postgres.has_access_doc(user[0], id)


@router.post("/{id}/access")
def add_access(id: str, aux: Access):
    postgres.give_access_doc(aux.username, id, aux.access_type)
    return JSONResponse(content={"detail": "Access updated"}, status_code=202)


@router.delete("/{id}/access/me")
def deny_access(id: str, user: tuple = Depends(get_current_user)):
    postgres.deny_access_doc(user[0], id)
    return JSONResponse(content={"detail": "Access deleted"}, status_code=201)


@router.delete("/{id}/access")
def deny_access(id: str, aux: Access):
    postgres.deny_access_doc(aux.username, id)
    return JSONResponse(content={"detail": "Access deleted"}, status_code=201)
