import json

from fastapi import APIRouter, Depends
from fastapi.responses import JSONResponse

from bsition.api.models.document import Access
from bsition.api.models.table import Filter, Sort, Table, Column
from bsition.api.utils.security import get_current_user
from bsition.backend.mongo import tables as mongo
from bsition.backend.postgres import relations as postgres_r
from bsition.backend.postgres import tables as postgres_t

router = APIRouter()


@router.post("")
def create_table(aux: Table):
    print(json.loads(json.dumps(aux.__dict__)))
    mongo.create_table({
        "_id": aux.id,
        "name": aux.name,
        "column_names": aux.column_names,
        "types": aux.types,
        "owner": aux.owner,
        "data": aux.data
    })
    return JSONResponse(content={"detail": "Table created."}, status_code=201)


@router.put("/{id}")
def edit_table(aux: Column, id: int):
    if aux.data is not None:
        mongo.edit_column(aux.row_number, aux.column, aux.data, id)
    if aux.column is not None and aux.data is None:
        mongo.insert_columns(id, aux.column, aux.type)
    return JSONResponse(content={"detail": "Table updated."}, status_code=202)


@router.put("/{id}/sort")
def add_sort(aux: Sort, id: str):
    postgres_t.add_sort(id, aux.property, aux.order, aux.priority)
    return JSONResponse(content={"detail": "Sort updated."}, status_code=202)


@router.get("/{id}/sort")
def sort(id: str):
    return mongo.sort(int(id))


@router.put("/{id}/filter")
def add_filter(aux: Filter, id: str):
    postgres_t.add_filter(id, aux.property, aux.value, aux.function)
    return JSONResponse(content={"detail": "Filter updated."}, status_code=202)


@router.get("/{id}/filter")
def filter(id: str):
    return mongo.filter(int(id))


@router.get("/{id}/access")
def get_access(id: str, user: tuple = Depends(get_current_user)):
    return postgres_r.has_access_table(user[0], id)


@router.post("/{id}/access")
def give_access(id: str, access_type: Access, user: tuple = Depends(get_current_user)):
    postgres_r.give_access_table(user[0], id, access_type.access_type)
    return JSONResponse(content={"detail": "Access created"}, status_code=201)


@router.put("/{id}/access")
def update_access(
    id: str, access_type: Access, user: tuple = Depends(get_current_user)
):
    postgres_r.give_access_table(user[0], id, access_type.access_type)
    return JSONResponse(content={"detail": "Access updated"}, status_code=202)


@router.delete("/{id}/access")
def deny_access(id: str, user: tuple = Depends(get_current_user)):
    postgres_r.deny_access_table(user[0], id)
    return JSONResponse(content={"detail": "Access deleted"}, status_code=201)
